﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KitaFormatter = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const templates_1 = require("./templates");
const plugin_1 = require("./templates/plugin");
const route_1 = require("./templates/route");
class KitaFormatter {
    config;
    typeOnly;
    files = [];
    constructor(config, typeOnly) {
        this.config = config;
        this.typeOnly = typeOnly;
    }
    generateRoute(route, collector) {
        this.files.push((0, route_1.generateRoute)(route, this.config.cwd, path_1.default.relative(this.config.cwd, this.config.src), collector.getProviders()));
    }
    generateRuntime(collector) {
        this.files.push((0, templates_1.generateIndex)(collector.getRoutes()));
        this.files.push((0, plugin_1.generatePlugin)(collector.getRoutes(), collector.getSchemas(), collector.getPlugins(), collector.getProviders(), path_1.default.relative(this.config.cwd, this.config.src)));
    }
    async flush() {
        const files = this.files.flatMap((file) => this.typeOnly ? [file.types] : this.config.declaration ? [file.source, file.types] : file.source);
        // Tries to delete previous index.js and index.d.ts original files
        // Ignores errors
        await Promise.allSettled([
            fs_1.default.promises.unlink(path_1.default.join(this.config.runtimePath, 'index.js')),
            fs_1.default.promises.unlink(path_1.default.join(this.config.runtimePath, 'index.d.ts'))
        ]);
        // Make sure all directories exists
        const dirs = files.map((f) => path_1.default.dirname(f.filename)).filter((v, i, a) => a.indexOf(v) === i);
        await Promise.all(dirs.map((dir) => fs_1.default.promises.mkdir(path_1.default.join(this.config.runtimePath, dir), {
            recursive: true
        })));
        // Write all files
        await Promise.all(files.map((file) => fs_1.default.promises.writeFile(path_1.default.join(this.config.runtimePath, file.filename), file.content)));
        // Clear files
        const length = this.files.length;
        this.files = [];
        return this.config.declaration ? length * 2 : length;
    }
}
exports.KitaFormatter = KitaFormatter;
//# sourceMappingURL=formatter.js.map