﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toLifecycleArray = exports.toReplacedSchema = exports.toAsyncStatement = exports.toParamHelper = exports.toOptions = exports.generateRoute = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@kitajs/common");
const json_stable_stringify_1 = tslib_1.__importDefault(require("json-stable-stringify"));
const path_1 = tslib_1.__importDefault(require("path"));
const ts_writer_1 = require("ts-writer");
const path_2 = require("../util/path");
function generateRoute(route, cwd, cwdSrcRelativity, providers) {
    const returnTypeName = (0, common_1.capital)(`${route.schema.operationId}Response`);
    return (0, ts_writer_1.ts) `${`routes/${route.schema.operationId}`}
  'use strict';

  const ${common_1.kControllerName} = require(${(0, path_2.toMaybeRelativeImport)(route.relativePath, cwdSrcRelativity)});

  ${route.parameters
        .flatMap((p) => p.imports)
        .concat(route.imports || [])
        .filter((imp) => !!imp)
        // Remove duplicates
        .filter((imp, index, arr) => arr.findIndex((i) => i.name === imp.name) === index)
        .map((r) => `const ${r.name} = require(${(0, path_2.toMaybeRelativeImport)(r.path, cwdSrcRelativity)});`)}
  
  exports.${route.schema.operationId} = ${common_1.kControllerName}.${route.controllerMethod}.bind(null);
  
  exports.${route.schema.operationId}Handler = ${toAsyncStatement(route.parameters)}function ${route.schema.operationId}Handler(${common_1.kRequestParam}, ${common_1.kReplyParam}) {
    ${route.parameters.map(toParamHelper)}
    ${route.customReturn ||
        `return exports.${route.schema.operationId}(${ts_writer_1.ts.join(route.parameters.map((p) => p.value), ',')});`}
  }

  ${route.method === 'ALL' ? `const { supportedMethods } = require('fastify/lib/httpMethods');` : ''}

  exports.${route.schema.operationId}Options = ${toOptions(route, providers)};

  exports.${route.schema.operationId}Url = ${JSON.stringify(route.url)};

  exports.${route.schema.operationId}Method = ${JSON.stringify(route.method)};

  exports.__esModule = true;

  ${ts_writer_1.ts.types}

  import type * as ${common_1.kControllerName} from '${(0, path_2.escapePath)((0, path_2.removeExt)(path_1.default.join(cwd, route.relativePath)))}';
  import type { FastifyRequest, FastifyReply } from 'fastify';

  /**
   * The controller method for the ${route.schema.operationId} route.
   */
  export declare const ${route.schema.operationId}: typeof ${common_1.kControllerName}.${route.controllerMethod};

  /**
   * The url for the ${route.schema.operationId} route.
   */
  export declare const ${route.schema.operationId}Url: string;

  /**
   * The method for the ${route.schema.operationId} route.
   */
  export declare const ${route.schema.operationId}Method: string;

  /**
   * The return type of the controller method.
   *
   * ${String(route.schema.description || '')}
   */
  export type ${returnTypeName} = Awaited<ReturnType<typeof ${route.schema.operationId}>>;

  /**
   * Parses the request and reply parameters and calls the ${route.schema.operationId} controller method.
   */
  export declare function ${route.schema.operationId}Handler(
    ${common_1.kRequestParam}: FastifyRequest,
    ${common_1.kReplyParam}: FastifyReply
  ): ${toAsyncStatement(route.parameters) ? `Promise<${returnTypeName}>` : returnTypeName};
  `;
}
exports.generateRoute = generateRoute;
function toOptions(r, providers) {
    const handler = `{
    url: '${r.url}',
    method: ${r.method === 'ALL' ? 'supportedMethods' : `'${r.method}'`},
    handler: exports.${r.schema.operationId}Handler,
    schema: ${toReplacedSchema(r)},
    ${toLifecycleArray(r.parameters, providers)}
 }`;
    return r.options ? r.options.replace('$1', handler) : handler;
}
exports.toOptions = toOptions;
function toParamHelper(param) {
    if (!param.helper) {
        return undefined;
    }
    return `${param.helper}
  
  // This helper may have already resolved this request
  if (${common_1.kReplyParam}.sent) {
    return reply;
  }`;
}
exports.toParamHelper = toParamHelper;
function toAsyncStatement(parameters) {
    if (!parameters.some((p) => p.helper?.includes('await'))) {
        return undefined;
    }
    return 'async ';
}
exports.toAsyncStatement = toAsyncStatement;
function toReplacedSchema(r) {
    let code = (0, json_stable_stringify_1.default)(r.schema, { space: 4 });
    for (const param of r.parameters) {
        if (param.schemaTransformer) {
            code = `${param.providerName}.transformSchema(${code}${Array.isArray(param.schemaTransformer) ? `, ${param.schemaTransformer.join(', ')}` : ''})`;
        }
    }
    return code;
}
exports.toReplacedSchema = toReplacedSchema;
function toLifecycleArray(parameters, providers) {
    const hookTypes = {};
    for (const parameter of parameters) {
        if (!parameter.providerName) {
            continue;
        }
        const provider = providers.find((p) => p.type === parameter.providerName);
        for (const hook of provider.lifecycleHooks) {
            (hookTypes[hook] ??= []).push(parameter.providerName);
        }
    }
    return Object.entries(hookTypes)
        .map(([hook, values]) => `${hook}: [${values.map((v) => `${v}.${hook}`).join(', ')}]`)
        .join(',');
}
exports.toLifecycleArray = toLifecycleArray;
//# sourceMappingURL=route.js.map