﻿"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.escapePath = exports.removeExt = exports.toMaybeRelativeImport = exports.isRelative = exports.PREVIOUS_DIR = exports.CURRENT_DIR = void 0;
const tslib_1 = require("tslib");
const common_1 = require("@kitajs/common");
const path_1 = tslib_1.__importDefault(require("path"));
/** Posix: `./`, in windows: `.\\` */
exports.CURRENT_DIR = `.${path_1.default.sep}`;
exports.PREVIOUS_DIR = `..${path_1.default.sep}`;
function isRelative(p) {
    if (p.startsWith(exports.CURRENT_DIR)) {
        return exports.CURRENT_DIR;
    }
    if (p.startsWith(exports.PREVIOUS_DIR)) {
        return exports.PREVIOUS_DIR;
    }
    return false;
}
exports.isRelative = isRelative;
/* Returns an already quoted import-ready string */
function toMaybeRelativeImport(imp, cwdSrcRelativity) {
    const withoutExt = removeExt(imp);
    const relative = isRelative(withoutExt);
    if (!relative) {
        return `'${escapePath(withoutExt)}'`;
    }
    // Removes source folder from path, as it will be contained in the `KITA_GLOBAL_ROOT` variable
    return `\`\${${common_1.kKitaGlobalRoot}}${escapePath(path_1.default.sep + path_1.default.relative(cwdSrcRelativity, withoutExt))}\``;
}
exports.toMaybeRelativeImport = toMaybeRelativeImport;
/** Removes the extension from a path, if present. */
function removeExt(p) {
    const ext = path_1.default.extname(p);
    // Only remove the extension if its present
    if (ext) {
        return p.slice(0, -ext.length);
    }
    else {
        return p;
    }
}
exports.removeExt = removeExt;
/** Matches all \ in a string */
const DOUBLE_BACKSLASH = /\\/g;
const DOLLAR_SIGN = /\$/g;
const SINGLE_QUOTE = /'/g;
const DOUBLE_QUOTE = /"/g;
/** Escape string for use inside path strings. */
function escapePath(p) {
    return p
        .replace(DOUBLE_BACKSLASH, '\\\\')
        .replace(DOLLAR_SIGN, '\\$')
        .replace(SINGLE_QUOTE, "\\'")
        .replace(DOUBLE_QUOTE, '\\"');
}
exports.escapePath = escapePath;
//# sourceMappingURL=path.js.map